# Copyright 2020, 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

package MyAppendMorphism;
use 5.010;
use strict;
use Carp 'croak';
use List::Util 'max','min','sum','pairmap';
use POSIX 'floor';
use Math::PlanePath::Base::Digits 'bit_split_lowtohigh';

use base 'Exporter';
our @EXPORT_OK = ('@HtoL_state_table',
                  'HtoL_FLAT',

                  '%sLtoH_state_table',
                  '@LtoH_state_table',
                  '$LtoH_starting_state','$LtoH_max_state',
                  '@LtoH_state_to_name','@LtoH_accepting_states',
                  '%LtoH_name_to_state',
                  'LtoH_FLAT',

                  'Mblock','@s_to_dx','@s_to_dy',
                 );
our %EXPORT_TAGS = (all => \@EXPORT_OK);

# uncomment this to run the ### lines
# use Smart::Comments;


#------------------------------------------------------------------------------
our @HtoL_state_table = ([0,0],   # dummy 0 state
                         [1,2],
                         [1,3],
                         [4,2],
                         [4,3],
                         [3,4],   # 5
                         [1,5],   # 6  start sharing state 1
                         [1,2],   # 7
                         [7,5],   # 8  start no share state 1
                        );
sub HtoL_FLAT {
  my ($accepting, $starting) = @_;
  $starting ||= 8;  # demanding 2 bits
  return MyFLAT::aref_to_FLAT_DFA
    (\@HtoL_state_table,
     name      => "HtoL a=$accepting starting $starting",
     starting  => $starting,
     accepting => $accepting);
}


#------------------------------------------------------------------------------

# symbols
our %sLtoH_state_table = ('2a'  => ['2a', '3a'],
                          '3a'  => ['2b', '3c'],
                          '4a'  => ['1b', '4c'],
                          '1a'  => ['1a', '4a'],
                          '2b'  => ['2a', '2c'],
                          '3g'  => ['1b', '4c'],
                          '4g'  => ['3d', '3b'],
                          '1b'  => ['1a', '1c'],
                          '2c'  => ['2b', '3c'],
                          '3b'  => ['2b', '2d'],
                          '4c'  => ['4d', '4b'],
                          '1c'  => ['1b', '4c'],
                          '2d'  => ['3d', '3b'],
                          '3c'  => ['3d', '3b'],
                          '4b'  => ['1b', '1d'],
                          '1d'  => ['4d', '4b'],
                          '2e'  => ['2b', '2d'],
                          '3d'  => ['3f', '3e'],
                          '4d'  => ['4f', '4e'],
                          '1e'  => ['1b', '1d'],
                          '2f'  => ['3d', '2e'],
                          '3e'  => ['3d', '2e'],
                          '4f'  => ['4f', '1f'],
                          '1f'  => ['4d', '1e'],
                          '3f'  => ['3f', '2f'],
                          '4e'  => ['4d', '1e'],
                          'T'  => ['1a', '3g'],
                          'U'  => ['2b', '4g'],
                          'S'  => ['T', 'U'],
                         );
our @LtoH_state_to_name =
  ('',
   '2a','3a','4a','1a','2b','3g','4g',
   '1b','2c','3b','4c','1c','2d','3c','4b','1d','2e','3d','4d',
   '1e','2f','3e','4f','1f',
   '',
   '3f','4e','T','U','S');
our $LtoH_max_state = $#LtoH_state_to_name;

our %LtoH_name_to_state;
foreach my $state (0 .. $LtoH_max_state) {
  if (my $name = $LtoH_state_to_name[$state]) {
    $LtoH_name_to_state{$name} = $state;
  }
}

our @LtoH_state_table;
foreach my $state (0 .. $LtoH_max_state) {
  if (my $from = $LtoH_state_to_name[$state]) {
    $LtoH_name_to_state{$LtoH_state_to_name[$state]} = $state;
    $LtoH_state_table[$state] = [ map {$LtoH_name_to_state{$_}} @{$sLtoH_state_table{$from}} ];
  } else {
    $LtoH_state_table[$state] = [0,0];     # dummy state
  }
}

our $LtoH_starting_state = ($LtoH_name_to_state{'S'} // die);
our @LtoH_accepting_states;
foreach my $a (1..4) {
  $LtoH_accepting_states[$a]
    = [grep {substr($LtoH_state_to_name[$_],0,1) eq $a} 0 .. $LtoH_max_state];
}

sub LtoH_FLAT {
  my ($accepting) = @_;
  ### $LtoH_starting_state
  return MyFLAT::aref_to_FLAT_DFA
    (\@LtoH_state_table,
     name      => "LtoH a=$accepting",
     starting  => $LtoH_starting_state,
     accepting_list => $LtoH_accepting_states[$accepting]);
}

#------------------------------------------------------------------------------

sub Mblock {
  my ($p) = @_;
  my @v = (1,2,3,4);
  foreach (1..$p) {
    @v = map {@{$HtoL_state_table[$_]}} @v;
  }
  return @v;
}

our @s_to_dx = (undef, 1, 0, 0, -1);
our @s_to_dy = (undef, 0, 1, -1, 0);


#------------------------------------------------------------------------------
1;
__END__
