#!/usr/bin/perl -w

# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

# Usage: perl gen-att.pl
#
# Create files
#
#     HtoL1.att
#     HtoL2.att
#     HtoL3.att
#     HtoL4.att
#     HtoL-final.att
#     LtoH1.att
#     LtoH2.att
#     LtoH3.att
#     LtoH4.att
#     LtoH-final.att

use 5.010;
use strict;
use warnings;
use File::Spec;

use FindBin;
use lib $FindBin::Bin;
use MyAppendMorphism ':all';
use MyFLAT;


my $epsilon = '@0@';   # epsilon transition symbol in the .att files
my $output_dir = $FindBin::Bin;

foreach my $accepting (1..4) {
  my $filename = File::Spec->catfile($output_dir,"HtoL$accepting.att");
  print "create $filename\n";
  open my $fh, '>', $filename or die;
  print $fh "0\t8\t$epsilon\t$epsilon\n";  # starting, demand 2 bits
  foreach my $from (1 .. $#HtoL_state_table) {
    foreach my $bit (0,1) {
      my $to = $HtoL_state_table[$from]->[$bit];
      print $fh "$from\t$to\t$bit\t$bit\n";
    }
  }
  print $fh "$accepting\n";  # accepting
  close $fh or die;
}
{
  # HtoL-final.att
  # epsilon transition from each accepting state to output a(n)

  my $filename = File::Spec->catfile($output_dir,"HtoL-final.att");
  print "create $filename\n";
  open my $fh, '>', $filename or die;
  print $fh "0\t6\t$epsilon\t$epsilon\n";  # starting
  foreach my $from (1 .. $#HtoL_state_table) {
    if ($from <= 4) {
      my $to = 6 + $from;
      print $fh "$from\t$to\t$epsilon\t$from\n";
      print $fh "$to\n";  # accepting
    }
    foreach my $bit (0,1) {
      my $to = $HtoL_state_table[$from]->[$bit];
      print $fh "$from\t$to\t$bit\t$epsilon\n";
    }
  }
  close $fh or die;
}

foreach my $accepting (1..4) {
  my $filename = File::Spec->catfile($output_dir,"LtoH$accepting.att");
  print "create $filename\n";
  open my $fh, '>', $filename or die;
  print $fh "0\t$LtoH_max_state\t$epsilon\t$epsilon\n";  # starting
  foreach my $from (1 .. $#LtoH_state_table) {
    foreach my $bit (0,1) {
      my $to = $LtoH_state_table[$from]->[$bit];
      print $fh "$from\t$to\t$bit\t$bit\n";
    }
  }
  foreach my $state (@{$LtoH_accepting_states[$accepting]}) {
    print $fh "$state\n";
  }
  close $fh or die;
}
{
  # LtoH-final.att
  # epsilon transition from each accepting state to output a(n)

  my $filename = File::Spec->catfile($output_dir,"LtoH-final.att");
  print "create $filename\n";
  open my $fh, '>', $filename or die;
  print $fh "0\t$LtoH_max_state\t$epsilon\t$epsilon\n";  # starting
  foreach my $from (1 .. $#LtoH_state_table) {
    foreach my $bit (0,1) {
      my $to = $LtoH_state_table[$from]->[$bit];
      print $fh "$from\t$to\t$bit\t$epsilon\n";
    }
    my $name = $LtoH_state_to_name[$from];
    if ($name =~ /^([1234])/) {
      my $ret = $1;
      my $to = $LtoH_max_state + $from;
      print $fh "$from\t$to\t$epsilon\t$ret\n";
      print $fh "$to\n";  # accepting
    }
  }
  close $fh or die;
}

exit 0;
